import Drawnings.DirectionType;
import Drawnings.DrawningCleaningCar;
import Drawnings.DrawningTruck;
import MovementStrategy.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Random;
import java.util.List;
import java.util.LinkedList;

public class FormCleaningCar extends JFrame {
    protected DrawningTruck _drawningTruck;
    private JPanel pictureBox;
    private JButton buttonRight;
    private JButton buttonLeft;
    private JButton buttonDown;
    private JButton buttonUp;
    private JComboBox comboBoxStrategy;
    private JButton buttonStrategyStep;
    private List<JComponent> controls;

    private AbstractStrategy _strategy;
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        if (_drawningTruck.getEntityTruck() == null ||
                pictureBox.getWidth() == 0 || pictureBox.getHeight() == 0)
            return;
        Graphics e = pictureBox.getGraphics();
        e.setColor(pictureBox.getBackground());
        e.fillRect(0,0, pictureBox.getWidth(), pictureBox.getHeight());
        _drawningTruck.DrawTransport(e);
        RepaintControls();
    }
    public void SetTruck(DrawningTruck truck) {
        _drawningTruck = truck;
        _drawningTruck.SetPictureSize(pictureBox.getWidth(), pictureBox.getHeight());
        comboBoxStrategy.setEnabled(true);
        _strategy = null;
        repaint(); //по-идее вызовет paint()
    }
    public FormCleaningCar() {
        setSize(1000, 600);
        setLocationRelativeTo(null);
        setTitle("Project Cleaning Car");
        add(pictureBox);
        setVisible(true);
        buttonUp.setName("buttonUp");
        buttonDown.setName("buttonDown");
        buttonLeft.setName("buttonLeft");
        buttonRight.setName("buttonRight");
        comboBoxStrategy.setEnabled(false);
        InitializeControlsRepaintList();
        ActionListener buttonMoveClickedListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String buttonName = ((JButton) e.getSource()).getName();
                boolean result = false;

                switch (buttonName) {
                    case "buttonUp": {
                        result = _drawningTruck.MoveTransport(DirectionType.Up);
                    }
                    break;
                    case "buttonDown": {
                        result = _drawningTruck.MoveTransport(DirectionType.Down);
                    }
                    break;
                    case "buttonLeft": {
                        result = _drawningTruck.MoveTransport(DirectionType.Left);
                    }
                    break;
                    case "buttonRight": {
                        result = _drawningTruck.MoveTransport(DirectionType.Right);
                    }
                    break;

                }
                if (result)
                    Draw();

            }
        };
        buttonStrategyStep.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_drawningTruck == null) {
                    return;
                }
                if (comboBoxStrategy.isEnabled()) {
                    switch (comboBoxStrategy.getSelectedIndex()) {
                        case 0:
                            _strategy = new MoveToCenter();
                            break;
                        case 1:
                            _strategy = new MoveToBorder();
                            break;
                        default:
                            _strategy = null;
                            break;
                    }
                    if (_strategy == null) {
                        return;
                    }
                    _strategy.SetData(new MoveableTruck(_drawningTruck), pictureBox.getWidth(), pictureBox.getHeight());
                }
                if (_strategy == null) {
                    return;
                }
                comboBoxStrategy.setEnabled(false);
                _strategy.MakeStep();
                Draw();

                if (_strategy.GetStatus() == StrategyStatus.Finish) {
                    comboBoxStrategy.setEnabled(true);
                    _strategy = null;
                }
            }
        });
        buttonRight.addActionListener(buttonMoveClickedListener);
        buttonDown.addActionListener(buttonMoveClickedListener);
        buttonLeft.addActionListener(buttonMoveClickedListener);
        buttonUp.addActionListener(buttonMoveClickedListener);


        comboBoxStrategy.addItem("К центру");
        comboBoxStrategy.addItem("К краю");
    }
    private void Draw() {
        if (_drawningTruck.getEntityTruck() == null ||
                pictureBox.getWidth() == 0 || pictureBox.getHeight() == 0)
            return;
        Graphics g = pictureBox.getGraphics();
        g.setColor(pictureBox.getBackground());
        g.fillRect(0,0, pictureBox.getWidth(), pictureBox.getHeight());
        _drawningTruck.DrawTransport(g);
        RepaintControls();

    }
    private void RepaintControls() {
        for (JComponent control : controls) {
            control.repaint();
        }
    }
    private void InitializeControlsRepaintList() {
        controls = new LinkedList<>();

        controls.add(buttonUp);
        controls.add(buttonDown);
        controls.add(buttonLeft);
        controls.add(buttonRight);

        controls.add(comboBoxStrategy);
        controls.add(buttonStrategyStep);
    }
}